
import java.io.File;
import java.io.IOException;
import java.util.HashMap;
import java.util.Iterator;

/**
 * 
 * @author Anny Marit Svendsen
 *
 */
public class LagVektorer {
	
	private double[][] docVectors;
	private double[][] repVectors;
	private BuildIndex build;
	private int antFiles, antRepFiles;
	private HashMap<String,IndexTermInfo> termIndex;
	
	/**			
	 * Konstruerer et objekt av type LagVektorer med matriser for dokumenter
	 * og klyngerepresentanter. For K-means (med klyngerepresentanter).
	 * 		
	 * @param 	files		filene som skal bli til vektorer.
	 * @param	repFiles	filene som er valgt (av bruker eller tilfeldig)
	 * 						til klyngerepresentanter.
	 * @param 	weightType	vektingtype som skal benyttes.
	 * @throws IOException
	 */
	public LagVektorer(File[] files, File[] repFiles, Weights weightType) throws IOException{
		antFiles = files.length;
		antRepFiles = repFiles.length;
		build = new BuildIndex();
		termIndex = build.getTermIndex();	
		mIndex(files, true);
		addRepFiles(repFiles);
		calcTermWeights(weightType);
		docVectors = new double[antFiles][];
		repVectors = new double[antRepFiles][];
		assignVectors(true);
	}
	
	/**
	 * Konstruerer et objekt av type LagVektorer med matrise for dokumenter.			
	 * For hierarkisk klyngeanalyse (uten klyngerepresentanter).
	 * 						
	 * @param 	files		filene som skal bli til vektorer.
	 * @throws IOException
	 */
	public LagVektorer(File[] files, Weights weightType) throws IOException{
		antFiles = files.length;
		build = new BuildIndex();
		termIndex = build.getTermIndex();
		mIndex(files, false);	
		calcTermWeights(weightType);
		docVectors = new double[antFiles][];
		assignVectors(false);	
	}
	
	/**
	 * Leser listen av filer og lager en indeks. Indeksen blir 
	 * lagret i en HashMap.
	 * 
	 * @param docs			filene som skal indekseres.
	 * @throws IOException
	 */
	private void mIndex(File[] docs, boolean kmeans) throws IOException{
		//read the list of files and make an index (hashmap - term, indexterminfo)	
		if (kmeans){
			for (int i=0; i<antFiles; i++){
				build.makeIndex(docs[i], i, antRepFiles+antFiles, true);
				System.out.println(docs[i]+" er indeksert for k-means");
			}
		}
		else{
			for (int i=0; i<antFiles; i++){
				build.makeIndex(docs[i], i, antFiles, true);
				System.out.println(docs[i]+" er indeksert for hierarkisk");
			}
		}
	}
	
	/**
	 * Legger til filene som er de initielle klyngerepresentantene.
	 * Filene blir lagt til i indeksen etter dokumentene som skal klynges 
	 * er lagt til.
	 * 
	 * @param 	docs	Filene som er satt til  vre initielle klyngerepresentanter.
	 */
	private void addRepFiles(File[] docs) throws IOException{
		//legger til repfilene etter de ordinre indeksfilene
		System.out.println("repfiles");
		for (int i=antFiles; i<antFiles+antRepFiles; i++){
			build.makeIndex(docs[i-antFiles], i, antFiles+antRepFiles, false);
			System.out.println(docs[i-antFiles]+" er indeksert");
		}
	}
	
	/**
	 * Henter ut vektene fra IndexTermInfo for hver term i hvert dokument,
	 * og bygger opp en dokumentvektor for hvert dokument. Hvis K-means
	 * skal benyttes blir ogs klyngerepresentantene bygd opp p samme
	 * mte.
	 * 
	 * @param 	kmeans	angir hvilken algoritme for klyngeanalysen som skal 
	 * 					benyttes. Variabelen er satt til true hvis det er 
	 * 					K-means og false hvis det er hierarkisk klyngeanalyse
	 * 					som skal benyttes.
	 */
	private void assignVectors(boolean kmeans){
		double[] vect;
		int count = 0;	
		System.out.println("size: "+termIndex.size());	
		
		String term ="";
		IndexTermInfo indexTermInfo;
		Iterator<String> iterator;
		
		for (int i=0; i<antFiles; i++){		
			count = 0;
			vect = new double[termIndex.keySet().size()];//ant termer
			
			//henter ut dokumentvektor
			iterator = termIndex.keySet().iterator();
			while (iterator.hasNext()){
				term = iterator.next();
				indexTermInfo = (IndexTermInfo) termIndex.get(term);
				vect[count] = (double) indexTermInfo.getWeight(i);
				count++;
			}
			docVectors[i] = vect;
			System.out.print(i+ " ");
		}
		System.out.println("--");

		if (kmeans){
			//adder filene for klyngerepresentanter bare hvis k-means skal benyttes
			for (int i=antFiles; i<antFiles+antRepFiles; i++){		
				count = 0;
				vect = new double[termIndex.keySet().size()];
				
				//henter ut dokumentvektor
				iterator = termIndex.keySet().iterator();
				while (iterator.hasNext()){
					term = iterator.next();
					indexTermInfo = (IndexTermInfo) termIndex.get(term);
					vect[count] = (double) indexTermInfo.getWeight(i);
					count++;
				}
				repVectors[i-antFiles] = vect;
				System.out.print(i+ " ");
			}		
		}			
		//sjekker arraylista
		/*for (int i=0; i<vectors.size(); i++){
			for (int j=0; j<((double[])vectors.get(i)).length; j++){
				System.out.print(((double[])vectors.get(i))[j]+" ");
			}
			System.out.println();
		}*/
		//return vectors;
	}
	
	/**
	 * Kalkulerer termvekter for hver term i hvert dokument.
	 * 
	 * @param weightType angir hvilken vektingtype som skal benyttes.
	 */
	private void calcTermWeights(Weights weightType){
		IndexTermInfo iTermInf;
		long t1, t2;
		int num = 0;
		
		System.out.println("Starting to calculate weights ...");
		System.out.println("Weightscheme used = " + weightType);
	
		t1 = System.currentTimeMillis();
		Iterator<IndexTermInfo> iterator = termIndex.values().iterator();	
		while (iterator.hasNext()){
			num++;
			iTermInf = iterator.next();
			iTermInf.setWeights(weightType);		
		}	
		t2 = System.currentTimeMillis();
	
		System.out.println("Finished calculating weights.");
		System.out.println("Time used = " + (t2-t1) + " msec");
	}
	
	/**
	 * Returnerer matrise som inneholder dokumentvektorene.
	 * 
	 * @return verdien av denne double[][].
	 */
	public double[][] getVectors(){
		return docVectors;
	}

	/**
	 * Returnerer matrise som inneholder klyngerepresentantvektorene.
	 * 
	 * @return verdien av denne double[][].
	 */
	public double[][] getRepVectors(){
		return repVectors;
	}
}
